@@ -112,8 +112,11 @@ static struct evtab_entry evtab[] = {
  		"button/volumeup VOLUP 00000080 00000000"},
 	{{{0,0}, EV_KEY, KEY_MUTE, 1},
  		"button/mute MUTE 00000080 00000000"},
+/* Kernel 3.1 or later required for KEY_MICMUTE */
+#ifdef KEY_MICMUTE
 	{{{0,0}, EV_KEY, KEY_MICMUTE, 1},
 		"button/micmute MICMUTE 00000080 00000000"},
+#endif
  	/* cd play/pause buttons */
  	{{{0,0}, EV_KEY, KEY_NEXTSONG, 1},
  		"cd/next CDNEXT 00000080 00000000"},
@@ -175,6 +178,8 @@ static struct evtab_entry evtab[] = {
 	{{{0,0}, EV_KEY, KEY_FN_F9, 1}, "button/fnf9 FNF9 00000080 00000000"},
 	{{{0,0}, EV_KEY, KEY_FN_F10, 1}, "button/fnf10 FF10 00000080 00000000"},
 	{{{0,0}, EV_KEY, KEY_FN_F11, 1}, "button/fnf11 FF11 00000080 00000000"},
+	/* F20 is sometimes used for micmute */
+	{{{0,0}, EV_KEY, KEY_F20, 1}, "button/f20 F20 00000080 00000000"},
 	/* Fn-F9 produces KEY_F24 on Thinkpad T42p */
 	{{{0,0}, EV_KEY, KEY_F24, 1}, "button/f24 F24 00000080 00000000"},
 	{{{0,0}, EV_KEY, KEY_KBDILLUMTOGGLE, 1},
@@ -201,6 +206,30 @@ static struct evtab_entry evtab[] = {
 
 };
 
+/* special support for the MUTE key, as the key toggles we want to
+ * consider repeated keys but don't report them all the time. We just
+ * ensure that the number of key presses (MOD 2) is correct.
+ */
+static const char *
+mute_string(struct input_event event)
+{
+	if (event.type == EV_KEY && event.code == KEY_MUTE) {
+		static size_t repeat_count;
+		if (event.value == 1) {
+			repeat_count = 1;
+			return "button/mute MUTE (key pressed)";
+		} else if (event.value == 2) {
+			repeat_count++;
+			return NULL;
+		} else if (event.value == 0) {
+			if (repeat_count % 2 == 0) {
+				return "button/mute MUTE (key released)";
+			}
+		}
+	}
+	return NULL;
+}
+
 /*----------------------------------------------------------------------*/
 /* Given an input event, returns the string corresponding to that event.
    If there is no corresponding string, NULL is returned.  */
@@ -299,7 +328,12 @@ static void process_input(int fd)
 	}
 	
 	/* convert the event into a string */
-	str = event_string(event);
+	if (tpmutefix) {
+		str = mute_string(event);
+		if (str ==  NULL)
+			str = event_string(event);
+	} else
+		str = event_string(event);
 	/* if this is not an event we care about, bail */
 	if (str == NULL)
 		return;
